@@ -26,6 +26,59 @@ COEF.polr     <- function( object, ... ) summary(object)$coefficients
 VCOV.gnlm     <- function( object, ... ) object$cov
 VCOV.rq       <- function( object, ... ) summary(object, cov=TRUE)$cov
 
+# Computing the contrasts between two prediction data frames
+# --- a utility called from ci.lin 
+ci.dfr <-
+function( obj, ndx, ndr, Exp )
+    {
+if( !is.data.frame(ndx) ) stop("2nd argument in list must be a data frame")
+if( !is.data.frame(ndr) ) stop("3rd argument in list must be a data frame")
+if( nrow(ndr)==1 ) ndr <- ndr[rep(1,nrow(ndx)),,drop=FALSE]
+if( (    ( nrow(ndx) !=  nrow(ndr)) ) |
+    ( any(names(ndx) != names(ndr)) ) )
+    stop("\nThe two prediction frames must have same dimensions and column names:",
+         "but dimensions are: ", dim(ndx), " and ", dim(ndr), "\n", 
+         "and column names are:\n",
+         "exp: ", names(ndx), "\n",
+         "ref: ", names(ndr), "\n")
+# Now fix those variable that are needed in order to get model.matrix working
+# Supplied variable names
+ cols <- names( ndx )
+# Factors in model; which are supplied; derive names of omitted
+ facs <- names(obj$xlevels)
+ofacs <- match( facs, cols )
+ofacs <- facs[is.na(ofacs)]
+# Variables in model; which are supplied; derive names of omitted
+ vars <- setdiff( all.vars(obj$formula)[-1], facs )
+ovars <- match( vars, cols )
+ovars <- vars[is.na(ovars)]
+# Construct the extra columns
+xcols <- ndx[,NULL]
+if( length(ofacs) > 0 ) for( fn in ofacs ) xcols <- cbind( xcols, obj$xlevels[[fn]][1] )
+if( length(ovars) > 0 ) for( vn in ovars ) xcols <- cbind( xcols, 1 )
+if( dim(xcols)[2]>0 )
+  {
+  names( xcols ) <- c(ofacs,ovars)
+  ndx <- cbind( ndx, xcols )
+  ndr <- cbind( ndr, xcols )
+  }
+# Factors must have more than one level, which they typically will not
+# have in the specification of the prediction frames, so we find the
+# factors in the model and expand levels to the complete set of levels
+dcl <- attr(obj$terms,"dataClasses")
+whf <- ( dcl == "factor" )
+if( any(whf) )
+  for( fn in names(dcl)[which(whf)] )
+     {
+     ndx[,fn] <- factor( ndx[,fn], levels=obj$xlevels[[fn]] )
+     ndr[,fn] <- factor( ndr[,fn], levels=obj$xlevels[[fn]] )
+     }
+# Then we can set up the contrast matrix and call ci.lin
+CM <- model.matrix( formula(obj)[-2], data=ndx ) -
+      model.matrix( formula(obj)[-2], data=ndr )
+ci.lin( obj, ctr.mat=CM, Exp=Exp )
+    }
+
 ci.lin <-
 function( obj,
       ctr.mat = NULL,
@@ -39,6 +92,15 @@ function( obj,
           Exp = FALSE,
        sample = FALSE )
 {
+# If ctr.mat is a list of two dataframes call ci.dfr
+if( is.list(ctr.mat) )
+  {
+  if( !is.data.frame(ctr.mat[[1]]) |
+      !is.data.frame(ctr.mat[[2]]) )
+      stop("If ctr.mat is a list it must be a list of two data frames")
+  return( ci.dfr( obj, ctr.mat[[1]], ctr.mat[[2]], Exp=Exp ) )
+  }
+  
 # First extract all the coefficients and the variance-covariance matrix
 cf  <- COEF( obj )
 vcv <- VCOV( obj )
@@ -202,7 +264,7 @@ if( !diffs )
     }
 # Return the requested structure
 if( sample ) invisible( res ) else
-if( vcov ) invisible( list( est=ct, vcov=vc ) ) else res
+if( vcov ) invisible( list( coef=ct[,1], vcov=vc ) ) else res
 }
 
 # Handy wrapper
